diff -x CVS -x fc.mnu -x emx.diff -uwrN ../OggVorbis.clean/ao/src/ao_wav.c ./ao/src/ao_wav.c
--- ../OggVorbis.clean/ao/src/ao_wav.c	Sun Dec 24 19:44:12 2000
+++ ./ao/src/ao_wav.c	Tue Dec 26 18:17:28 2000
@@ -29,6 +29,7 @@
 #include <errno.h>
 #include <string.h>
 #include <unistd.h>
+#include <sys/types.h>
 #include <fcntl.h>
 #include <signal.h>
 #include <ao/ao.h>
diff -x CVS -x fc.mnu -x emx.diff -uwrN ../OggVorbis.clean/ao/src/audio_out.c ./ao/src/audio_out.c
--- ../OggVorbis.clean/ao/src/audio_out.c	Sat Dec 23 00:14:30 2000
+++ ./ao/src/audio_out.c	Sat Dec 23 01:04:48 2000
@@ -103,6 +103,7 @@
 	struct stat statbuf;
 	void *plughand;
 	char fullpath[FILENAME_MAX];
+	char *plugin_path;
 
 	if (driver_head == NULL) {
 		/* insert the null and wav drivers into the tree */
@@ -120,10 +121,16 @@
 		driver = dwav;
 
 		/* now insert any plugins we find */
-		plugindir = opendir(AO_PLUGIN_PATH);
+		plugin_path = getenv("AO_PLUGIN_PATH");
+
+		if (plugin_path == NULL) {
+			plugin_path = AO_PLUGIN_PATH;
+		}
+
+		plugindir = opendir(plugin_path);
 		if (plugindir != NULL) {
 			while ((plugin_dirent = readdir(plugindir)) != NULL) {
-				snprintf(fullpath, FILENAME_MAX, "%s/%s", AO_PLUGIN_PATH, plugin_dirent->d_name);
+				snprintf(fullpath, FILENAME_MAX, "%s/%s", plugin_path, plugin_dirent->d_name);
 				if (!stat(fullpath, &statbuf) && S_ISREG(statbuf.st_mode) && (ext = strrchr(plugin_dirent->d_name, '.')) != NULL) {
 					if (strcmp(ext, SHARED_LIB_EXT) == 0) {
 						plugin = _get_plugin(fullpath);
diff -x CVS -x fc.mnu -x emx.diff -uwrN ../OggVorbis.clean/ao/src/emx.mak ./ao/src/emx.mak
--- ../OggVorbis.clean/ao/src/emx.mak	Thu Jan  1 00:00:00 1970
+++ ./ao/src/emx.mak	Tue Dec 19 01:38:54 2000
@@ -0,0 +1,10 @@
+CC = gcc
+CFLAGS=-O6 -Ios2 -I../include -DAO_PLUGIN_PATH=\".\" -DSHARED_LIB_EXT=\".dll\"
+
+OBJ=audio_out.o ao_null.o ao_wav.o os2/dl_os2.o
+
+ao.a: $(OBJ)
+	ar cr $@ $(OBJ)
+
+.c.o: .AUTODEPEND
+	$(CC) -c $(CFLAGS) $< -o $*.o
diff -x CVS -x fc.mnu -x emx.diff -uwrN ../OggVorbis.clean/ao/src/os2/dl_os2.c ./ao/src/os2/dl_os2.c
--- ../OggVorbis.clean/ao/src/os2/dl_os2.c	Thu Jan  1 00:00:00 1970
+++ ./ao/src/os2/dl_os2.c	Tue Dec 26 18:12:24 2000
@@ -0,0 +1,53 @@
+#define INCL_DOS
+#include <os2.h>
+#include <dlfcn.h>
+#include <stdio.h>
+
+char message[500];
+ULONG rc;
+
+void *dlopen(const char *filename, int flag)
+{
+	char failed_module[20];
+	HMODULE handle;
+
+	if ((rc = DosLoadModule(failed_module, sizeof(failed_module), (PCHAR)filename, &handle)) != 0) {
+		sprintf(message, "Error %u loading %s (failed module %s)", rc, filename, failed_module);
+		return NULL;
+	}
+
+	return (void *)handle;
+}
+
+
+
+const char *dlerror()
+{
+	if ( rc == 0 )
+		return NULL;
+	return message;
+}
+
+
+
+void *dlsym(void *handle, const char *symbol)
+{
+	PFN func;
+	CHAR name[100];
+
+	strcpy( name, symbol );
+
+	if ((rc = DosQueryProcAddr((HMODULE)handle, 0, name, &func)) != 0) {
+		sprintf(message, "Error %u loading symbol %s", rc, name);
+		return NULL;
+	}
+
+	return func;
+}
+
+
+
+int dlclose(void *handle)
+{
+	return DosFreeModule((HMODULE)handle);
+}
diff -x CVS -x fc.mnu -x emx.diff -uwrN ../OggVorbis.clean/ao/src/os2/dlfcn.h ./ao/src/os2/dlfcn.h
--- ../OggVorbis.clean/ao/src/os2/dlfcn.h	Thu Jan  1 00:00:00 1970
+++ ./ao/src/os2/dlfcn.h	Mon Dec 18 13:18:00 2000
@@ -0,0 +1,17 @@
+#ifndef __dlfcn_os2_h
+#define __dlfcn_os2_h
+
+/* For dlopen () */
+#define RTLD_LAZY	1
+#define RTLD_NOW	2
+#define RTLD_GLOBAL	0x100
+
+/* For dlsym */
+#define RTLD_NEXT	((void *)-1)
+
+void *dlopen(const char *filename, int flag);
+const char *dlerror();
+void *dlsym(void *handle, const char *symbol);
+int dlclose(void *handle);
+
+#endif
diff -x CVS -x fc.mnu -x emx.diff -uwrN ../OggVorbis.clean/ao/src/os2/os_types.h ./ao/src/os2/os_types.h
--- ../OggVorbis.clean/ao/src/os2/os_types.h	Thu Jan  1 00:00:00 1970
+++ ./ao/src/os2/os_types.h	Tue Dec 26 18:22:10 2000
@@ -0,0 +1,34 @@
+/*
+ *
+ *  os_types.h
+ *
+ *      Original Copyright (C) Aaron Holtzman - May 1999
+ *      Modifications Copyright (C) Stan Seibert - July 2000
+ *
+ *  This file is part of libao, a cross-platform audio output library.  See
+ *  README for a history of this source code.
+ *
+ *  libao is free software; you can redistribute it and/or modify
+ *  it under the terms of the GNU General Public License as published by
+ *  the Free Software Foundation; either version 2, or (at your option)
+ *  any later version.
+ *
+ *  libao is distributed in the hope that it will be useful,
+ *  but WITHOUT ANY WARRANTY; without even the implied warranty of
+ *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ *  GNU General Public License for more details.
+ *
+ *  You should have received a copy of the GNU General Public License
+ *  along with GNU Make; see the file COPYING.  If not, write to
+ *  the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
+ *
+ */
+
+/* Set type sizes for this platform */
+
+typedef unsigned char     uint_8;
+typedef unsigned short    uint_16;
+typedef unsigned long     uint_32;
+typedef signed   char     sint_32;
+typedef signed   short    sint_16;
+typedef signed   long     sint_8;
diff -x CVS -x fc.mnu -x emx.diff -uwrN ../OggVorbis.clean/ao/src/plugins/os2/ao_os2.c ./ao/src/plugins/os2/ao_os2.c
--- ../OggVorbis.clean/ao/src/plugins/os2/ao_os2.c	Thu Jan  1 00:00:00 1970
+++ ./ao/src/plugins/os2/ao_os2.c	Tue Dec 26 18:24:56 2000
@@ -0,0 +1,175 @@
+#include "Dart.h"
+#include <process.h>
+#include <ao/ao.h>
+
+#define WipeVar(var) memset(&var, 0, sizeof(var))
+
+ao_info_t ao_os2_info =
+{
+	"OS/2 audio driver output ",
+	"dart",
+	"Brian Havard <brianh@kheldar.apana.org.au>",
+	"Outputs audio to the OS/2 DART interface."
+};
+
+ao_info_t *plugin_get_driver_info(void)
+{
+	return &ao_os2_info;
+}
+
+
+typedef struct {
+	USHORT usDeviceID;
+	MCI_MIXSETUP_PARMS MixSetupParms;
+	MCI_BUFFER_PARMS BufferParms;
+	MCI_MIX_BUFFER *BufferList;
+	int NumBuffers;
+	int CurBuf, BufPos;
+	int FreeBuffers;
+	HEV BufferFreed;
+} ao_internal_s;
+
+
+
+long DARTEventCallback(ULONG ulStatus, PMCI_MIX_BUFFER pBuffer, ULONG ulFlags)
+{
+	ao_internal_s *state = (ao_internal_s *)(pBuffer->ulUserParm);
+
+	if (ulFlags & MIX_WRITE_COMPLETE) {
+		DosPostEventSem(state->BufferFreed);
+	}
+
+	return TRUE;
+}
+
+
+
+ao_internal_t *plugin_open(uint_32 bits, uint_32 rate, uint_32 channels, ao_option_t *options)
+{
+	int b;
+	ao_internal_s *state = (ao_internal_s *)calloc(1, sizeof(*state));
+
+	// First open the ampmix device
+	MCI_OPEN_PARMS OpenParms;
+	ULONG rc;
+	WipeVar(OpenParms);
+	DosCreateEventSem(NULL, &state->BufferFreed, 0, 0);
+	OpenParms.pszDeviceType = (PSZ)MAKEULONG(MCI_DEVTYPE_AUDIO_AMPMIX,1);
+	rc = mciSendCommand(0, MCI_OPEN, MCI_WAIT | MCI_OPEN_TYPE_ID | MCI_OPEN_SHAREABLE, &OpenParms, 0);
+
+	if (rc != MCIERR_SUCCESS)
+		return NULL;
+
+	state->usDeviceID = OpenParms.usDeviceID;
+
+	// Next setup the DART parameters
+	WipeVar(state->MixSetupParms);
+	state->MixSetupParms.ulBitsPerSample = bits;
+	state->MixSetupParms.ulFormatTag     = MCI_WAVE_FORMAT_PCM;
+	state->MixSetupParms.ulSamplesPerSec = rate;
+	state->MixSetupParms.ulChannels = channels;    /* Stereo */
+	state->MixSetupParms.ulFormatMode = MCI_PLAY;
+	state->MixSetupParms.ulDeviceType = MCI_DEVTYPE_WAVEFORM_AUDIO;
+	state->MixSetupParms.pmixEvent = DARTEventCallback;
+
+	rc = mciSendCommand(state->usDeviceID,
+						 MCI_MIXSETUP,
+						 MCI_WAIT | MCI_MIXSETUP_INIT,
+						 &state->MixSetupParms,
+						 0);
+
+	if (rc != MCIERR_SUCCESS)
+		return NULL;
+
+	// Allocate DART buffers
+	WipeVar(state->BufferParms);
+	state->BufferParms.ulBufferSize = state->MixSetupParms.ulBufferSize;
+	state->BufferParms.ulNumBuffers = state->MixSetupParms.ulNumBuffers * 2;
+	state->BufferList = (MCI_MIX_BUFFER *)calloc(state->BufferParms.ulNumBuffers, sizeof(MCI_MIX_BUFFER));
+	state->BufferParms.pBufList = state->BufferList;
+	state->NumBuffers = state->BufferParms.ulNumBuffers;
+
+	rc = mciSendCommand(state->usDeviceID,
+						 MCI_BUFFER,
+						 MCI_WAIT | MCI_ALLOCATE_MEMORY,
+						 &state->BufferParms,
+						 0);
+
+	if (rc != MCIERR_SUCCESS)
+		return NULL;
+
+	for (b=0; b < state->NumBuffers; b++) {
+		state->BufferList[b].ulUserParm = (ULONG)state;
+	}
+
+	state->CurBuf = state->BufPos = 0;
+	state->FreeBuffers = state->MixSetupParms.ulNumBuffers;
+	DosSetPriority(PRTYS_THREAD, PRTYC_TIMECRITICAL, 30, 0);
+	return state;
+}
+
+
+
+void plugin_close(ao_internal_t *vstate)
+{
+	ao_internal_s *state = (ao_internal_s *)vstate;
+
+	if (state->BufPos > 0) { // Take care of the leftovers
+		memset(state->BufferList[state->CurBuf].pBuffer + state->BufPos, 0, state->MixSetupParms.ulBufferSize - state->BufPos);
+		state->FreeBuffers--;
+		state->MixSetupParms.pmixWrite(state->MixSetupParms.ulMixHandle, state->BufferList + state->CurBuf, 1);
+	}
+
+	while (state->FreeBuffers < state->MixSetupParms.ulNumBuffers) {
+		ULONG count;
+		DosWaitEventSem(state->BufferFreed, -1);
+		DosResetEventSem(state->BufferFreed, &count);
+		state->FreeBuffers += count;
+	}
+
+	mciSendCommand(state->usDeviceID,
+					MCI_BUFFER,
+					MCI_WAIT | MCI_DEALLOCATE_MEMORY,
+					&state->BufferParms,
+					0);
+
+	mciSendCommand(state->usDeviceID, MCI_CLOSE, MCI_WAIT, NULL, 0);
+	DosSetPriority(PRTYS_THREAD, PRTYC_REGULAR, -30, 0);
+	DosCloseEventSem(state->BufferFreed);
+}
+
+
+
+void plugin_play(ao_internal_t *vstate, void* output_samples, uint_32 bytes)
+{
+	ao_internal_s *state = (ao_internal_s *)vstate;
+	const char *buffer = (const char *)output_samples;
+	int space, transferbytes;
+
+	while (bytes > 0) {
+		while (state->FreeBuffers == 0) {
+			ULONG count;
+			DosWaitEventSem(state->BufferFreed, -1);
+			DosResetEventSem(state->BufferFreed, &count);
+			state->FreeBuffers += count;
+		}
+
+		space = state->MixSetupParms.ulBufferSize - state->BufPos;
+		transferbytes = bytes <= space ? bytes : space;
+
+		memcpy(state->BufferList[state->CurBuf].pBuffer + state->BufPos, buffer, transferbytes);
+		state->BufPos += transferbytes;
+		bytes -= transferbytes;
+		buffer += transferbytes;
+
+		if (state->BufPos == state->MixSetupParms.ulBufferSize) {
+			state->FreeBuffers--;
+			state->MixSetupParms.pmixWrite(state->MixSetupParms.ulMixHandle, state->BufferList + state->CurBuf, 1);
+			state->CurBuf++;
+			state->BufPos = 0;
+
+			if (state->CurBuf == state->MixSetupParms.ulNumBuffers)
+				state->CurBuf = 0;
+		}
+	}
+}
diff -x CVS -x fc.mnu -x emx.diff -uwrN ../OggVorbis.clean/ao/src/plugins/os2/dart.def ./ao/src/plugins/os2/dart.def
--- ../OggVorbis.clean/ao/src/plugins/os2/dart.def	Thu Jan  1 00:00:00 1970
+++ ./ao/src/plugins/os2/dart.def	Sat Dec 23 01:06:30 2000
@@ -0,0 +1,7 @@
+LIBRARY dart INITINSTANCE
+EXPORTS
+	plugin_get_driver_info
+	plugin_open
+	plugin_close
+	plugin_play
+	DARTEventCallback
diff -x CVS -x fc.mnu -x emx.diff -uwrN ../OggVorbis.clean/ao/src/plugins/os2/Dart.h ./ao/src/plugins/os2/Dart.h
--- ../OggVorbis.clean/ao/src/plugins/os2/Dart.h	Thu Jan  1 00:00:00 1970
+++ ./ao/src/plugins/os2/Dart.h	Tue Dec 26 18:15:40 2000
@@ -0,0 +1,65 @@
+#ifndef __Dart_h
+#define __Dart_h
+
+#define INCL_DOS
+#include <os2.h>
+
+#define INCL_MCIOS2
+#include <os2me.h>
+
+#define MCI_BUFFER   62
+#define MCI_MIXSETUP 63
+#define MCI_MIXSETUP_QUERYMODE 0x40000
+#define MCI_MIXSETUP_INIT      0x10000
+#define MCI_ALLOCATE_MEMORY    0x40000
+#define MCI_DEALLOCATE_MEMORY  0x80000
+#define MIX_BUFFER_EOS         0x00001
+#define MIX_STREAM_ERROR       0x00080
+#define MIX_READ_COMPLETE      0x00001
+#define MIX_WRITE_COMPLETE     0x00002
+
+typedef struct _MCI_MIX_BUFFER {
+	ULONG  ulStructLength;  /* Length of the structure */
+	PCHAR  pBuffer;         /* Pointer to a buffer */
+	ULONG  ulBufferLength;  /* Length of the buffer */
+	ULONG  ulFlags;         /* Flags */
+	ULONG  ulUserParm;      /* User buffer parameter */
+	ULONG  ulTime;          /* Device time in milliseconds */
+	ULONG  ulReserved1;     /* Unused */
+	ULONG  ulReserved2;     /* Unused */
+} MCI_MIX_BUFFER;
+
+typedef struct _MCI_BUFFER_PARMS {
+	HWND   hwndCallback;    /* Window for notifications */
+	ULONG  ulStructLength;  /* Length of MCI_BUFFER_PARMS */
+	ULONG  ulNumBuffers;    /* Number of buffers to allocate (IN/OUT)*/
+	ULONG  ulBufferSize;    /* Size of buffers mixer should use */
+	ULONG  ulMintoStart;    /* Unused */
+	ULONG  ulSrcStart;      /* Unused */
+	ULONG  ulTgtStart;      /* Unused */
+	MCI_MIX_BUFFER *pBufList; /* Pointer to array of buffers */
+} MCI_BUFFER_PARMS;
+
+typedef MCI_MIX_BUFFER *PMCI_MIX_BUFFER;
+typedef long (*PMIXERPROC)( ULONG ulMixHandle, PMCI_MIX_BUFFER pBuffer, ULONG ulBuffers );
+typedef long (*PMIXEREVENT)( ULONG ulStatus, PMCI_MIX_BUFFER pBuffer, ULONG ulFlags );
+
+typedef struct _MCI_MIXSETUP_PARMS
+{
+	HWND        hwndCallback;    /* IN  Window for notifications */
+	ULONG       ulBitsPerSample; /* IN  Number of bits per sample */
+	ULONG       ulFormatTag;     /* IN  Format tag */
+	ULONG       ulSamplesPerSec;  /* IN  Sampling rate */
+	ULONG       ulChannels;      /* IN  Number of channels */
+	ULONG       ulFormatMode;    /* IN  MCI_RECORD or MCI_PLAY */
+	ULONG       ulDeviceType;    /* IN  MCI_DEVTYPE */
+	ULONG       ulMixHandle;     /* OUT Read/Write handle */
+	PMIXERPROC  pmixWrite;       /* OUT Write routine entry point */
+	PMIXERPROC  pmixRead;        /* OUT Read routine entry point */
+	PMIXEREVENT pmixEvent;       /* IN  Event routine entry point */
+	PVOID       pExtendedInfo;   /* IN  Media-specific info */
+	ULONG       ulBufferSize;    /* OUT Recommended buffer size */
+	ULONG       ulNumBuffers;    /* OUT Recommended num buffers */
+} MCI_MIXSETUP_PARMS;
+
+#endif
diff -x CVS -x fc.mnu -x emx.diff -uwrN ../OggVorbis.clean/ao/src/plugins/os2/emx.mak ./ao/src/plugins/os2/emx.mak
--- ../OggVorbis.clean/ao/src/plugins/os2/emx.mak	Thu Jan  1 00:00:00 1970
+++ ./ao/src/plugins/os2/emx.mak	Tue Dec 26 18:27:28 2000
@@ -0,0 +1,10 @@
+CC = gcc
+CFLAGS=-O6 -Zdll -Zcrtdll -I../../os2 -I../../../include
+
+OBJ=ao_os2.o
+
+dart.dll: $(OBJ)
+	gcc -Zdll -Zcrtdll -o $@ $(OBJ) -lmmpm2 dart.def
+
+.c.o:
+	$(CC) -c $(CFLAGS) $<
